==> gridstack-8.x-2.9.tar.gz: gridstack/src/GridStackManager.php <==
<?php

namespace Drupal\gridstack;

use Drupal\Core\Render\Element;
use Drupal\blazy\Blazy;
use Drupal\blazy\BlazyManagerBase;
use Drupal\gridstack\Entity\GridStack;
use Drupal\gridstack\Engine\GridStackEngineManagerInterface;
use Drupal\gridstack\Skin\GridStackSkinManagerInterface;
use Drupal\gridstack\Style\GridStackStylizerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides GridStack manager.
 */
class GridStackManager extends BlazyManagerBase implements GridStackManagerInterface {

  /**
   * The GridStack layout engine.
   *
   * @var \Drupal\gridstack\Engine\GridStackEnginePluginInterface
   */
  protected $engine;

  /**
   * The GridStack layout engine plugin ID.
   *
   * @var string
   */
  protected $engineId;

  /**
   * The GridStack optionset.
   *
   * @var \Drupal\gridstack\Entity\GridStack
   */
  protected $optionset;

  /**
   * The GridStack layout engine manager service.
   *
   * @var \Drupal\gridstack\Engine\GridStackEngineManagerInterface
   */
  protected $engineManager;

  /**
   * The GridStack skin manager service.
   *
   * @var \Drupal\gridstack\Skin\GridStackSkinManagerInterface
   */
  protected $skinManager;

  /**
   * The GridStack stylizer service.
   *
   * @var \Drupal\gridstack\Style\GridStackStylizerInterface
   */
  protected $stylizer;

  /**
   * The gridstack layout CSS classes.
   *
   * @var array
   */
  protected $mergedClasses;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->setEngineManager($container->get('gridstack.engine_manager'));
    $instance->setSkinManager($container->get('gridstack.skin_manager'));
    $instance->setStylizer($container->get('gridstack.stylizer'));
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderGridStack'];
  }

  /**
   * Returns GridStack layout engine manager service.
   */
  public function engineManager() {
    return $this->engineManager;
  }

  /**
   * Sets GridStack layout engine manager service.
   */
  public function setEngineManager(GridStackEngineManagerInterface $engine_manager) {
    $this->engineManager = $engine_manager;
    return $this;
  }

  /**
   * Returns GridStack skin manager service.
   */
  public function skinManager() {
    return $this->skinManager;
  }

  /**
   * Sets GridStack skin manager service.
   */
  public function setSkinManager(GridStackSkinManagerInterface $skin_manager) {
    $this->skinManager = $skin_manager;
    return $this;
  }

  /**
   * Returns GridStack stylizer service.
   */
  public function stylizer() {
    return $this->stylizer;
  }

  /**
   * Sets GridStack stylizer service.
   */
  public function setStylizer(GridStackStylizerInterface $stylizer) {
    $this->stylizer = $stylizer;
    return $this;
  }

  /**
   * Returns GridStack ayout engine plugin ID.
   */
  public function getEngineId() {
    if (!isset($this->engineId)) {
      $this->engineId = 'gridstack_js';
    }
    return $this->engineId;
  }

  /**
   * Sets GridStack layout engine plugin ID.
   */
  protected function setEngineId($engine_id) {
    $this->engineId = $engine_id;
    return $this;
  }

  /**
   * Returns the layout engine.
   */
  public function getEngine(array $settings, $engine = NULL) {
    if (!isset($this->engine) || $engine) {
      if (empty($engine)) {
        $engine = $settings['_engine'] ?? NULL;
        $engine = $engine ?: $this->getEngineId();
      }
      $this->engine = $this->engineManager->load($engine, $settings);
    }
    return $this->engine;
  }

  /**
   * {@inheritdoc}
   */
  public function attach(array $attach = []) {
    $load = parent::attach($attach);

    if (isset($this->engineManager)) {
      $this->engineManager->attach($load, $attach);
      $this->skinManager->attach($load, $attach);
      $this->stylizer->attach($load, $attach);
    }

    $this->moduleHandler->alter('gridstack_attach', $load, $attach);
    return $load;
  }

  /**
   * {@inheritdoc}
   */
  public function getMergedClasses($flatten = FALSE) {
    if (!isset($this->mergedClasses[$flatten])) {
      $classes = $this->engineManager->getClassOptions();
      $this->mergedClasses[$flatten] = $this->stylizer->getMergedClasses($flatten, $classes);
    }
    return $this->mergedClasses[$flatten];
  }

  /**
   * Prepares the engine settings.
   */
  public function engineSettings(array &$settings, $optionset = NULL) {
    $this->optionset = $optionset ?: $this->optionset;
    $this->prepareSettings($settings);
  }

  /**
   * Prepares the HTML settings.
   */
  public function prepareSettings(array &$settings) {
    $this->verify($settings);

    $blazies     = $settings['blazies'];
    $gridstacks  = $settings['gridstacks'];
    $optionset   = $this->optionset;
    $framework   = $settings['framework'] ?? '';
    $engine      = $settings['_engine'] ?? '';
    $gridnative  = $settings['gridnative'] ?? '';
    $column      = $optionset->getSetting('column', 12);
    $ungridstack = $settings['ungridstack'] ?? FALSE;
    $ungridstack = $gridstacks->is('ungridstack', $ungridstack);

    // The use_nested is not for js or native Grid, _yet, but CSS framework.
    $use_framework = $framework && $optionset->getOption('use_framework');
    $use_native    = !$use_framework && $gridnative;

    // Provides configurable layout engines.
    if ($use_framework) {
      $settings['background'] = $use_gridstack = $use_js = FALSE;
      $engine = $framework;
    }
    else {
      // Provides programmatic layout engines.
      $use_js = TRUE;
      $use_gridstack = !$ungridstack;
      $fallback = $use_native ? 'gridstack_native' : 'gridstack_js';
      $engine = $engine ?: $fallback;
    }

    $id = Blazy::getHtmlId('gridstack-' . $settings['optionset'] . '-' . $engine, $settings['id'] ?? '');

    $blazies->set('css.id', $id);

    $gridstacks->set('column', $column)
      ->set('engine', $engine)
      ->set('is.ungridstack', $ungridstack)
      ->set('use.framework', $use_framework)
      ->set('use.gridstack', $use_gridstack)
      ->set('use.gridnative', $use_native)
      ->set('use.js', $use_js)
      ->set('use.nested', $use_framework)
      ->set('was.settings', TRUE);

    // @todo remove after migration.
    $settings['column'] = $column;
    $settings['_engine'] = $engine;
    $settings['id'] = $id;
    $settings['_gridstack'] = $use_gridstack;
    $settings['gridnative'] = $use_native;
    $settings['use_js'] = $use_js;
    $settings['use_nested'] = $settings['use_framework'] = $use_framework;

    $this->setEngineId($engine);
  }

  /**
   * Returns the wrapper attributes, empty to allow overrides.
   */
  public function prepareAttributes(array &$build) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $build = []) {
    foreach (['attached', 'layout', 'variant'] + GridStackDefault::themeProperties() as $key) {
      $build[$key] = $build[$key] ?? [];
    }

    $gridstack = [
      '#theme'      => 'gridstack',
      '#build'      => $build,
      '#pre_render' => [[$this, 'preRenderGridStack']],
      'items'       => [],
      '#layout'     => $build['layout'],
    ];

    $this->moduleHandler->alter('gridstack_build', $gridstack, $build['settings']);
    return empty($build['items']) ? [] : $gridstack;
  }

  /**
   * {@inheritdoc}
   */
  public function preRenderGridStack($element) {
    $build = $element['#build'];
    unset($element['#build']);

    // Build GridStack elements.
    $this->prepareBuild($build, $element);
    $settings = &$build['settings'];

    // Provides cache, attributes, assets.
    // @todo reuse ::setAttachments post blazy:2.1+.
    $cache       = $this->getCacheMetadata($build);
    $attributes  = $this->prepareAttributes($build);
    $attachments = $this->attach($settings);
    $attachments = $this->merge($attachments, $build['attached'] ?? []);

    // Adds the required elements for the template.
    $element['#optionset']  = $this->optionset;
    $element['#settings']   = $this->merge($settings, $element, '#settings');
    $element['#postscript'] = $this->merge($build['postscript'], $element, '#postscript');
    $element['#attributes'] = $this->merge($attributes, $element, '#attributes');
    $element['#attached']   = $this->merge($attachments, $element, '#attached');
    $element['#cache']      = $this->merge($cache, $element, '#cache');

    // Layout Builder is happy, safe to free up wasted children.
    foreach (Element::children($element) as $child) {
      unset($element[$child]);
    }

    unset($build);
    return $element;
  }

  /**
   * Provides layout editor settings.
   */
  protected function adminSettings(array &$settings, array $element): void {
    $gridstacks = $settings['gridstacks'];
    $attributes = $element['#attributes'] ?? [];
    $prefix     = $element['#prefix'] ?? '';
    $id1        = 'data-layout-update-url';
    $id2        = 'data-layout-builder-highlight-id';
    $lb         = isset($attributes[$id1]) || isset($attributes[$id2]);
    $lbux       = $lb && $this->moduleHandler->moduleExists('lb_ux');
    $panels     = $prefix && strpos($prefix, 'panels-ipe-content') !== FALSE;

    // Detects layout builder, lbux, or panels.
    $infos = [
      'lb'     => $lb,
      'lbux'   => $lbux,
      'panels' => $panels,
      'ipe'    => $lb || $lbux || $panels,
    ];

    foreach ($infos as $key => $value) {
      $gridstacks->set('is.' . $key, $value);

      // @todo remove after migration.
      $settings['_' . $key] = $value;
    }
  }

  /**
   * Returns the common settings inherited down to each item.
   */
  protected function getGridStackSettings() {
    return array_intersect_key($this->configLoad('', 'gridstack.settings'), GridStackDefault::cleanUiSettings());
  }

  /**
   * Prepares the layout engine.
   */
  protected function initEngine(array &$build, array &$element) {
    $settings = &$build['settings'];

    $this->setEngine($settings);
    $this->engine->build($build, $element);
  }

  /**
   * Prepares GridStack build.
   */
  protected function prepareBuild(array &$build, array &$element) {
    // Just in case removed or changed to data-layout-builder-BLAH.
    $settings = &$build['settings'];

    $this->verify($settings);

    // Supports Blazy multi-breakpoint images if provided.
    if ($image = ($build['items'][0] ?? NULL)) {
      if (is_array($image)) {
        $this->isBlazy($settings, $image);
      }
    }

    // Provides the optionset if empty. GridStackLayout, or GridStackViews, may
    // already set this. GridStackFormatter used to set it, too.
    GridStack::verifyOptionset($build, $settings['optionset']);
    $this->optionset = $build['optionset'];

    // Prepares the settings, and init layout engine.
    $this->adminSettings($settings, $element);
    $this->engineSettings($settings);
    $this->initEngine($build, $element);
  }

  /**
   * Sets the layout engine.
   */
  protected function setEngine(array $settings) {
    $this->engine = $this->engineManager->load($settings['_engine'], $settings);
    return $this;
  }

  /**
   * Verifies `gridstacks` exists since few may be called outside the workflow.
   */
  public function verify(array &$settings): void {
    $settings += array_merge(GridStackDefault::htmlSettings(), $this->getGridStackSettings());
  }

  /**
   * Merge data with a new one with an optional key.
   *
   * @todo use Blazy::merge() post Blazy:2.10.
   */
  private function merge(array $data, array $element, $key = NULL): array {
    return GridStack::merge($data, $element, $key);
  }

}

==> gridstack-8.x-2.10.tar.gz: gridstack/src/GridStackManager.php <==
<?php

namespace Drupal\gridstack;

use Drupal\Core\Render\Element;
use Drupal\blazy\Blazy;
use Drupal\blazy\BlazyManagerBase;
use Drupal\blazy\BlazySettings;
use Drupal\gridstack\Entity\GridStack;
use Drupal\gridstack\Engine\GridStackEngineManagerInterface;
use Drupal\gridstack\Skin\GridStackSkinManagerInterface;
use Drupal\gridstack\Style\GridStackStylizerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides GridStack manager.
 */
class GridStackManager extends BlazyManagerBase implements GridStackManagerInterface {

  /**
   * The GridStack layout engine.
   *
   * @var \Drupal\gridstack\Engine\GridStackEnginePluginInterface
   */
  protected $engine;

  /**
   * The GridStack layout engine plugin ID.
   *
   * @var string
   */
  protected $engineId;

  /**
   * The GridStack optionset.
   *
   * @var \Drupal\gridstack\Entity\GridStack
   */
  protected $optionset;

  /**
   * The GridStack layout engine manager service.
   *
   * @var \Drupal\gridstack\Engine\GridStackEngineManagerInterface
   */
  protected $engineManager;

  /**
   * The GridStack skin manager service.
   *
   * @var \Drupal\gridstack\Skin\GridStackSkinManagerInterface
   */
  protected $skinManager;

  /**
   * The GridStack stylizer service.
   *
   * @var \Drupal\gridstack\Style\GridStackStylizerInterface
   */
  protected $stylizer;

  /**
   * The gridstack layout CSS classes.
   *
   * @var array
   */
  protected $mergedClasses;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->setEngineManager($container->get('gridstack.engine_manager'));
    $instance->setSkinManager($container->get('gridstack.skin_manager'));
    $instance->setStylizer($container->get('gridstack.stylizer'));
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderGridStack'];
  }

  /**
   * Returns GridStack layout engine manager service.
   */
  public function engineManager() {
    return $this->engineManager;
  }

  /**
   * Sets GridStack layout engine manager service.
   */
  public function setEngineManager(GridStackEngineManagerInterface $engine_manager) {
    $this->engineManager = $engine_manager;
    return $this;
  }

  /**
   * Returns GridStack skin manager service.
   */
  public function skinManager() {
    return $this->skinManager;
  }

  /**
   * Sets GridStack skin manager service.
   */
  public function setSkinManager(GridStackSkinManagerInterface $skin_manager) {
    $this->skinManager = $skin_manager;
    return $this;
  }

  /**
   * Returns GridStack stylizer service.
   */
  public function stylizer() {
    return $this->stylizer;
  }

  /**
   * Sets GridStack stylizer service.
   */
  public function setStylizer(GridStackStylizerInterface $stylizer) {
    $this->stylizer = $stylizer;
    return $this;
  }

  /**
   * Returns GridStack ayout engine plugin ID.
   */
  public function getEngineId() {
    if (!isset($this->engineId)) {
      $this->engineId = 'gridstack_js';
    }
    return $this->engineId;
  }

  /**
   * Sets GridStack layout engine plugin ID.
   */
  protected function setEngineId($engine_id) {
    $this->engineId = $engine_id;
    return $this;
  }

  /**
   * Returns the layout engine.
   */
  public function getEngine(array $settings, $engine = NULL) {
    if (!isset($this->engine) || $engine) {
      if (empty($engine)) {
        $config = $settings['gridstacks'];
        $engine = $config->get('engine');
        $engine = $engine ?: $this->getEngineId();
      }
      $this->engine = $this->engineManager->load($engine, $settings);
    }
    return $this->engine;
  }

  /**
   * {@inheritdoc}
   */
  public function attach(array $attach = []) {
    $load = parent::attach($attach);

    $this->verify($attach);

    if (isset($this->engineManager)) {
      $this->engineManager->attach($load, $attach);
      $this->skinManager->attach($load, $attach);
      $this->stylizer->attach($load, $attach);
    }

    $this->moduleHandler->alter('gridstack_attach', $load, $attach);
    return $load;
  }

  /**
   * {@inheritdoc}
   */
  public function getMergedClasses($flatten = FALSE) {
    if (!isset($this->mergedClasses[$flatten])) {
      $classes = $this->engineManager->getClassOptions();
      $this->mergedClasses[$flatten] = $this->stylizer->getMergedClasses($flatten, $classes);
    }
    return $this->mergedClasses[$flatten];
  }

  /**
   * Prepares the engine settings.
   */
  public function engineSettings(array &$settings, $optionset = NULL) {
    $this->optionset = $optionset ?: $this->optionset;
    $this->prepareSettings($settings);
  }

  /**
   * Prepares the HTML settings.
   */
  public function prepareSettings(array &$settings) {
    $this->verify($settings);

    $blazies     = $settings['blazies'];
    $gridstacks  = $settings['gridstacks'];
    $optionset   = $this->optionset;
    $framework   = $settings['framework'] ?? '';
    $engine      = $settings['_engine'] ?? '';
    $engine      = $engine ?: $gridstacks->get('engine');
    $gridnative  = $settings['gridnative'] ?? '';
    $gridnative  = $gridnative ?: $gridstacks->get('use.gridnative');
    $column      = $optionset->getSetting('column', 12);
    $ungridstack = $settings['ungridstack'] ?? FALSE;
    $ungridstack = $ungridstack ?: $gridstacks->is('ungridstack');

    // The use_nested is not for js or native Grid, _yet, but CSS framework.
    $use_framework = $framework && $optionset->getOption('use_framework');
    $use_native    = !$use_framework && $gridnative;

    // Provides configurable layout engines.
    if ($use_framework) {
      $settings['background'] = $use_gridstack = $use_js = FALSE;
      $engine = $framework;
    }
    else {
      // Provides programmatic layout engines.
      $use_js = TRUE;
      $use_gridstack = !$ungridstack;
      $fallback = $use_native ? 'gridstack_native' : 'gridstack_js';
      $engine = $engine ?: $fallback;
    }

    $id = Blazy::getHtmlId('gridstack-' . $settings['optionset'] . '-' . $engine, $settings['id'] ?? '');

    $blazies->set('css.id', $id);

    $gridstacks->set('id', $id)
      ->set('column', $column)
      ->set('engine', $engine)
      ->set('is.debug', $gridstacks->get('ui.debug'))
      ->set('is.ungridstack', $ungridstack)
      ->set('use.framework', $use_framework)
      ->set('use.gridstack', $use_gridstack)
      ->set('use.gridnative', $use_native)
      ->set('use.gridstatic', $gridstacks->get('ui.gridstatic'))
      ->set('use.js', $use_js)
      ->set('use.nested', $use_framework)
      ->set('was.settings', TRUE);

    // @todo remove after migration.
    $settings['column'] = $column;
    $settings['_engine'] = $engine;
    $settings['id'] = $id;
    $settings['_gridstack'] = $use_gridstack;
    $settings['gridnative'] = $use_native;
    $settings['use_js'] = $use_js;
    $settings['use_nested'] = $settings['use_framework'] = $use_framework;

    $this->setEngineId($engine);
  }

  /**
   * Returns the wrapper attributes, empty to allow overrides.
   */
  public function prepareAttributes(array &$build) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $build): array {
    foreach (['attached', 'layout', 'variant'] + GridStackDefault::themeProperties() as $key) {
      $build[$key] = $build[$key] ?? [];
    }

    $gridstack = [
      '#theme'      => 'gridstack',
      '#build'      => $build,
      '#pre_render' => [[$this, 'preRenderGridStack']],
      'items'       => [],
      '#layout'     => $build['layout'],
    ];

    $this->moduleHandler->alter('gridstack_build', $gridstack, $build['settings']);
    return empty($build['items']) ? [] : $gridstack;
  }

  /**
   * {@inheritdoc}
   */
  public function preRenderGridStack($element) {
    $build = $element['#build'];
    unset($element['#build']);

    // Build GridStack elements.
    $this->prepareBuild($build, $element);
    $settings = &$build['settings'];

    // Provides cache, attributes, assets.
    // @todo reuse ::setAttachments post blazy:2.1+.
    $cache       = $this->getCacheMetadata($build);
    $attributes  = $this->prepareAttributes($build);
    $attachments = $this->attach($settings);
    $attachments = Blazy::merge($attachments, $build['attached'] ?? []);

    // Adds the required elements for the template.
    $element['#optionset']  = $this->optionset;
    $element['#settings']   = Blazy::merge($settings, $element, '#settings');
    $element['#postscript'] = Blazy::merge($build['postscript'], $element, '#postscript');
    $element['#attributes'] = Blazy::merge($attributes, $element, '#attributes');
    $element['#attached']   = Blazy::merge($attachments, $element, '#attached');
    $element['#cache']      = Blazy::merge($cache, $element, '#cache');

    // Layout Builder is happy, safe to free up wasted children.
    foreach (Element::children($element) as $child) {
      unset($element[$child]);
    }

    unset($build);
    return $element;
  }

  /**
   * Reset the BlazySettings per item.
   */
  public function reset(array &$settings): BlazySettings {
    $this->verify($settings);

    // The settings instance must be unique per item.
    $object = $settings['gridstacks'];
    if (!$object->was('reset')) {
      $object->reset($settings);
      $object->set('was.reset', TRUE);
    }

    return $object;
  }

  /**
   * Verifies `gridstacks` exists since few may be called outside the workflow.
   */
  public function verify(array &$settings): void {
    if (!isset($settings['gridstacks'])) {
      $ui = $this->getUiSettings();
      // @todo remove merge.
      $settings += array_merge(GridStackDefault::htmlSettings(), $ui);
      $gridstacks = &$settings['gridstacks'];
      $gridstacks->set('ui', $ui);
    }
  }

  /**
   * Provides layout editor settings.
   */
  protected function adminSettings(array &$settings, array $element): void {
    $gridstacks = &$settings['gridstacks'];
    $attributes = $element['#attributes'] ?? [];
    $prefix     = $element['#prefix'] ?? '';
    $id1        = 'data-layout-update-url';
    $id2        = 'data-layout-builder-highlight-id';
    $lb         = isset($attributes[$id1]) || isset($attributes[$id2]);
    $lbux       = $lb && $this->moduleHandler->moduleExists('lb_ux');
    $panels     = $prefix && strpos($prefix, 'panels-ipe-content') !== FALSE;

    // Detects layout builder, lbux, or panels.
    $infos = [
      'lb'     => $lb,
      'lbux'   => $lbux,
      'panels' => $panels,
      'ipe'    => $lb || $lbux || $panels,
    ];

    foreach ($infos as $key => $value) {
      $gridstacks->set('is.' . $key, $value);

      // @todo remove after migration.
      $settings['_' . $key] = $value;
    }
  }

  /**
   * Returns the common settings inherited down to each item.
   */
  protected function getUiSettings() {
    return array_intersect_key($this->configLoad('', 'gridstack.settings'), GridStackDefault::cleanUiSettings());
  }

  /**
   * Prepares the layout engine.
   */
  protected function initEngine(array &$build, array &$element) {
    $settings = &$build['settings'];
    $gridstacks = $settings['gridstacks'];

    $this->setEngine($settings, $gridstacks->get('engine'));
    $this->engine->build($build, $element);
  }

  /**
   * Prepares GridStack build.
   */
  protected function prepareBuild(array &$build, array &$element) {
    // Just in case removed or changed to data-layout-builder-BLAH.
    $settings = &$build['settings'];

    $this->verify($settings);

    // Supports Blazy multi-breakpoint images if provided.
    if ($image = ($build['items'][0] ?? NULL)) {
      if (is_array($image)) {
        $this->isBlazy($settings, $image);
      }
    }

    // Provides the optionset if empty. GridStackLayout, or GridStackViews, may
    // already set this. GridStackFormatter used to set it, too.
    GridStack::verifyOptionset($build, $settings['optionset']);
    $this->optionset = $build['optionset'];

    // Prepares the settings, and init layout engine.
    $this->adminSettings($settings, $element);
    $this->engineSettings($settings);
    $this->initEngine($build, $element);
  }

  /**
   * Sets the layout engine.
   */
  protected function setEngine(array $settings, $engine = 'gridstack_js') {
    $this->engine = $this->engineManager->load($engine, $settings);
    return $this;
  }

}
